
Import interfaces
Import vm

Class TypeOfDelegate Implements IVMDelegate
	Method Exec:VMValue(scope:VMScope, params:VMValue[])
		If params.Length = 1 And params[0] <> Null
			Return VMValue.CreateString(TypeToText(params[0].type))
		End
		Return VMValue.CreateString("undefined")
	End
	
	Function TypeToText:String( type:Int )
		Select type
			Case 0 Return "number"
			Case 1 Return "string"
			Case 2 Return "func"
			Case 3 Return "internal func"
			Case 4 Return "internal object"
			Case 5 Return "scope"
			Case 6 Return "special"
			Case 7 Return "null"
			Case 8 Return "array"
			Case 9 Return "object"
		End
		Return "undefined"
	End
	
	Function Instance:VMValue()
		If g_Instance = Null
			g_Instance = VMValue.CreateDelegate(New TypeOfDelegate())
		End
		Return g_Instance
	End
	
	Global g_Instance:VMValue
End

Class PrintDelegate Implements IVMDelegate
	Method Exec:VMValue(scope:VMScope, params:VMValue[])
		#If TARGET = "html5" Or TARGET = "flash"
		Local out:String
		For Local i:Int = 0 Until params.Length
			If params[i].type = VMValue.TYPE_FLOAT
				out += String(params[i].dataFloat) + " "
			ElseIf params[i].type = VMValue.TYPE_STRING
				out += params[i].dataStr + " "
				' my eyes
			ElseIf params[i].type = VMValue.TYPE_SCOPE And params[i].dataScope <> Null And
					VMScope.ValueNotNull(params[i].dataScope.GetVariable("toString")) And
					params[i].dataScope.GetVariable("toString").type = VMValue.TYPE_FUNC
				out += params[i].dataScope.Call("toString", []).dataStr + " "
			Else
				out += "(" + TypeOfDelegate.TypeToText(params[i].type) + ") "
			End
		Next
		If out.Length Then Print out[..-1] Else Print ""
		#Else
		Local sb:StringStack = New StringStack()
		For Local i:Int = 0 Until params.Length
			If params[i].type = VMValue.TYPE_FLOAT
				sb.Push String(params[i].dataFloat)
			ElseIf params[i].type = VMValue.TYPE_STRING
				sb.Push params[i].dataStr
				' my eyes again
			ElseIf params[i].type = VMValue.TYPE_SCOPE And params[i].dataScope <> Null And
					VMScope.ValueNotNull(params[i].dataScope.GetVariable("toString")) And
					params[i].dataScope.GetVariable("toString").type = VMValue.TYPE_FUNC
				sb.Push params[i].dataScope.Call("toString", []).dataStr
			Else
				sb.Push "(" + TypeOfDelegate.TypeToText(params[i].type) + ")"
			End
		Next
		Print sb.Join(" ")
		#End
		Return Null
	End
	
	Function Instance:VMValue()
		If g_Instance = Null
			g_Instance = VMValue.CreateDelegate(New PrintDelegate())
		End
		Return g_Instance
	End
	
	Global g_Instance:VMValue
End

Class VMValueStack Extends Stack<VMValue> Implements IVMObjectInterface
	Method New( data:VMValue[] )
		Super.New(data)
	End
	
	Method Equals:Bool( lhs:VMValue, rhs:VMValue )
		
	End
	
	Method VMSet:VMValue( scope:VMScope, varName:String, data:VMValue )
		Select varName
			Case "length"
				Local currLen:Int = Length
				Length = Int(data.dataFloat)
				If currLen < Length
					For Local i:Int = currLen Until Length
						Set(i, VMValue.CreateNull())
					Next
				End
		End
		
		Return Null
	End
	
	Method VMGet:VMValue( scope:VMScope, varName:String )
		Select varName
			Case "length"	Return VMValue.CreateFloat(Length)
			Case "top"	Return Top()
		End
		
		Return Null
	End
	
	Method VMCall:VMValue( scope:VMScope, varName:String, params:VMValue[] )
		Select varName
			Case "clear"	Clear()
			Case "isEmpty"	Return VMValue.CreateFloat(IsEmpty())
			Case "contains"	Return VMValue.CreateFloat(Contains(params[0]))
			Case "push"		Push(params[0])
			Case "pop"		Return Pop()
			Case "insert"	Insert(params[0].dataFloat, params[1])
			Case "remove"	Remove(params[0].dataFloat)
		End
		
		Return Null
	End
End
